function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-disable no-param-reassign */

import { GraphQLID, GraphQLNonNull } from 'graphql';
import { fromGlobalId } from './globalId';
import NodeInterface from './nodeInterface';

import { getProjectionFromAST } from 'graphql-compose';

// this fieldConfig must be set to RootQuery.node field
export function getNodeFieldConfig(typeToFindByIdMap) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: NodeInterface,
    args: {
      id: {
        type: new GraphQLNonNull(GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = fromGlobalId(args.id);

      var type = _fromGlobalId.type;
      var id = _fromGlobalId.id;


      var findById = typeToFindByIdMap[type];
      if (findById && findById.resolve) {
        // suppose that first argument is argument with id field
        var idArgName = Object.keys(findById.args)[0];
        return findById.resolve({
          source: source,
          args: _defineProperty({}, idArgName, id), // eg. mongoose has _id fieldname, so should map
          context: context,
          info: info,
          projection: getProjectionFromAST(info)
        }).then(function (res) {
          if (res) {
            res.__nodeType = findById.getTypeComposer().getType();
          }
          return res;
        });
      }

      return null;
    }
  };
}