'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/core-js/object/define-property');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNodeFieldConfig = getNodeFieldConfig;

var _graphql = require('graphql');

var _globalId = require('./globalId');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

var _graphqlCompose = require('graphql-compose');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { (0, _defineProperty3.default)(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
/* eslint-disable no-param-reassign */

// this fieldConfig must be set to RootQuery.node field
function getNodeFieldConfig(typeToFindByIdMap) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: _nodeInterface2.default,
    args: {
      id: {
        type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = (0, _globalId.fromGlobalId)(args.id);

      var type = _fromGlobalId.type;
      var id = _fromGlobalId.id;


      var findById = typeToFindByIdMap[type];
      if (findById && findById.resolve) {
        // suppose that first argument is argument with id field
        var idArgName = (0, _keys2.default)(findById.args)[0];
        return findById.resolve({
          source: source,
          args: _defineProperty({}, idArgName, id), // eg. mongoose has _id fieldname, so should map
          context: context,
          info: info,
          projection: (0, _graphqlCompose.getProjectionFromAST)(info)
        }).then(function (res) {
          if (res) {
            res.__nodeType = findById.getTypeComposer().getType();
          }
          return res;
        });
      }

      return null;
    }
  };
}